//*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.tool;

import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.OperationApplicationException;
import android.database.Cursor;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.ui.NotesListAdapter.AppWidgetAttribute;

import java.util.ArrayList;
import java.util.HashSet;

/**
 * 该类提供了一系列用于操作笔记数据的工具方法，包括批量删除笔记、移动笔记到文件夹、获取文件夹数量等。
 */
public class DataUtils {
    // 日志标签，用于在日志中标识该类的输出信息
    public static final String TAG = "DataUtils";

    /**
     * 批量删除笔记的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param ids 要删除的笔记的 ID 集合
     * @return 如果删除成功返回 true，否则返回 false
     */
    public static boolean batchDeleteNotes(ContentResolver resolver, HashSet<Long> ids) {
        // 检查传入的 ID 集合是否为空
        if (ids == null) {
            // 如果为空，记录日志并返回 true
            Log.d(TAG, "the ids is null");
            return true;
        }
        // 检查 ID 集合的大小是否为 0
        if (ids.size() == 0) {
            // 如果为 0，记录日志并返回 true
            Log.d(TAG, "no id is in the hashset");
            return true;
        }

        // 创建一个用于存储内容提供者操作的列表
        ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();
        // 遍历 ID 集合
        for (long id : ids) {
            // 检查是否为系统根文件夹的 ID，如果是则不进行删除操作并记录错误日志
            if(id == Notes.ID_ROOT_FOLDER) {
                Log.e(TAG, "Don't delete system folder root");
                continue;
            }
            // 创建一个删除操作的构建器
            ContentProviderOperation.Builder builder = ContentProviderOperation
                    .newDelete(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id));
            // 将构建好的删除操作添加到操作列表中
            operationList.add(builder.build());
        }
        try {
            // 应用批量操作并获取操作结果
            ContentProviderResult[] results = resolver.applyBatch(Notes.AUTHORITY, operationList);
            // 检查操作结果是否为空或无效
            if (results == null || results.length == 0 || results[0] == null) {
                // 如果无效，记录日志并返回 false
                Log.d(TAG, "delete notes failed, ids:" + ids.toString());
                return false;
            }
            // 操作成功，返回 true
            return true;
        } catch (RemoteException e) {
            // 捕获远程异常并记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        } catch (OperationApplicationException e) {
            // 捕获操作应用异常并记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        }
        // 发生异常，返回 false
        return false;
    }

    /**
     * 将单个笔记移动到指定文件夹的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param id 要移动的笔记的 ID
     * @param srcFolderId 笔记的源文件夹 ID
     * @param desFolderId 笔记要移动到的目标文件夹 ID
     */
    public static void moveNoteToFoler(ContentResolver resolver, long id, long srcFolderId, long desFolderId) {
        // 创建一个 ContentValues 对象，用于存储要更新的字段和值
        ContentValues values = new ContentValues();
        // 设置笔记的父文件夹 ID 为目标文件夹 ID
        values.put(NoteColumns.PARENT_ID, desFolderId);
        // 设置笔记的原始父文件夹 ID 为源文件夹 ID
        values.put(NoteColumns.ORIGIN_PARENT_ID, srcFolderId);
        // 设置笔记的本地修改标志为 1
        values.put(NoteColumns.LOCAL_MODIFIED, 1);
        // 执行更新操作，将笔记移动到目标文件夹
        resolver.update(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id), values, null, null);
    }

    /**
     * 批量将笔记移动到指定文件夹的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param ids 要移动的笔记的 ID 集合
     * @param folderId 笔记要移动到的目标文件夹 ID
     * @return 如果移动成功返回 true，否则返回 false
     */
    public static boolean batchMoveToFolder(ContentResolver resolver, HashSet<Long> ids,
            long folderId) {
        // 检查传入的 ID 集合是否为空
        if (ids == null) {
            // 如果为空，记录日志并返回 true
            Log.d(TAG, "the ids is null");
            return true;
        }

        // 创建一个用于存储内容提供者操作的列表
        ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();
        // 遍历 ID 集合
        for (long id : ids) {
            // 创建一个更新操作的构建器
            ContentProviderOperation.Builder builder = ContentProviderOperation
                    .newUpdate(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id));
            // 设置笔记的父文件夹 ID 为目标文件夹 ID
            builder.withValue(NoteColumns.PARENT_ID, folderId);
            // 设置笔记的本地修改标志为 1
            builder.withValue(NoteColumns.LOCAL_MODIFIED, 1);
            // 将构建好的更新操作添加到操作列表中
            operationList.add(builder.build());
        }

        try {
            // 应用批量操作并获取操作结果
            ContentProviderResult[] results = resolver.applyBatch(Notes.AUTHORITY, operationList);
            // 检查操作结果是否为空或无效
            if (results == null || results.length == 0 || results[0] == null) {
                // 如果无效，记录日志并返回 false
                Log.d(TAG, "delete notes failed, ids:" + ids.toString());
                return false;
            }
            // 操作成功，返回 true
            return true;
        } catch (RemoteException e) {
            // 捕获远程异常并记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        } catch (OperationApplicationException e) {
            // 捕获操作应用异常并记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        }
        // 发生异常，返回 false
        return false;
    }

    /**
     * 获取除系统文件夹外的所有用户文件夹数量的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @return 用户文件夹的数量
     */
    public static int getUserFolderCount(ContentResolver resolver) {
        // 查询笔记数据库，获取符合条件的文件夹数量
        Cursor cursor =resolver.query(Notes.CONTENT_NOTE_URI,
                new String[] { "COUNT(*)" },
                NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>?",
                new String[] { String.valueOf(Notes.TYPE_FOLDER), String.valueOf(Notes.ID_TRASH_FOLER)},
                null);

        // 初始化文件夹数量为 0
        int count = 0;
        // 检查游标是否不为空
        if(cursor != null) {
            // 将游标移动到第一行
            if(cursor.moveToFirst()) {
                try {
                    // 从游标中获取文件夹数量
                    count = cursor.getInt(0);
                } catch (IndexOutOfBoundsException e) {
                    // 捕获索引越界异常并记录错误日志
                    Log.e(TAG, "get folder count failed:" + e.toString());
                } finally {
                    // 关闭游标
                    cursor.close();
                }
            }
        }
        // 返回文件夹数量
        return count;
    }

    /**
     * 检查笔记是否在数据库中可见的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param noteId 要检查的笔记的 ID
     * @param type 笔记的类型
     * @return 如果笔记可见返回 true，否则返回 false
     */
    public static boolean visibleInNoteDatabase(ContentResolver resolver, long noteId, int type) {
        // 查询笔记数据库，检查笔记是否存在且不在回收站中
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId),
                null,
                NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER,
                new String [] {String.valueOf(type)},
                null);

        // 初始化存在标志为 false
        boolean exist = false;
        // 检查游标是否不为空
        if (cursor != null) {
            // 检查游标中的记录数量是否大于 0
            if (cursor.getCount() > 0) {
                // 如果大于 0，说明笔记存在，设置存在标志为 true
                exist = true;
            }
            // 关闭游标
            cursor.close();
        }
        // 返回存在标志
        return exist;
    }

    /**
     * 检查笔记是否存在于数据库中的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param noteId 要检查的笔记的 ID
     * @return 如果笔记存在返回 true，否则返回 false
     */
    public static boolean existInNoteDatabase(ContentResolver resolver, long noteId) {
        // 查询笔记数据库，检查笔记是否存在
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId),
                null, null, null, null);

        // 初始化存在标志为 false
        boolean exist = false;
        // 检查游标是否不为空
        if (cursor != null) {
            // 检查游标中的记录数量是否大于 0
            if (cursor.getCount() > 0) {
                // 如果大于 0，说明笔记存在，设置存在标志为 true
                exist = true;
            }
            // 关闭游标
            cursor.close();
        }
        // 返回存在标志
        return exist;
    }

    /**
     * 检查数据是否存在于数据库中的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param dataId 要检查的数据的 ID
     * @return 如果数据存在返回 true，否则返回 false
     */
    public static boolean existInDataDatabase(ContentResolver resolver, long dataId) {
        // 查询数据数据库，检查数据是否存在
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_DATA_URI, dataId),
                null, null, null, null);

        // 初始化存在标志为 false
        boolean exist = false;
        // 检查游标是否不为空
        if (cursor != null) {
            // 检查游标中的记录数量是否大于 0
            if (cursor.getCount() > 0) {
                // 如果大于 0，说明数据存在，设置存在标志为 true
                exist = true;
            }
            // 关闭游标
            cursor.close();
        }
        // 返回存在标志
        return exist;
    }

    /**
     * 检查可见文件夹名称是否已存在的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param name 要检查的文件夹名称
     * @return 如果文件夹名称已存在返回 true，否则返回 false
     */
    public static boolean checkVisibleFolderName(ContentResolver resolver, String name) {
        // 查询笔记数据库，检查可见文件夹名称是否已存在
        Cursor cursor = resolver.query(Notes.CONTENT_NOTE_URI, null,
                NoteColumns.TYPE + "=" + Notes.TYPE_FOLDER +
                " AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER +
                " AND " + NoteColumns.SNIPPET + "=?",
                new String[] { name }, null);
        // 初始化存在标志为 false
        boolean exist = false;
        // 检查游标是否不为空
        if(cursor != null) {
            // 检查游标中的记录数量是否大于 0
            if(cursor.getCount() > 0) {
                // 如果大于 0，说明文件夹名称已存在，设置存在标志为 true
                exist = true;
            }
            // 关闭游标
            cursor.close();
        }
        // 返回存在标志
        return exist;
    }

    /**
     * 获取指定文件夹下的笔记小部件属性集合的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param folderId 文件夹的 ID
     * @return 笔记小部件属性的集合，如果没有则返回 null
     */
    public static HashSet<AppWidgetAttribute> getFolderNoteWidget(ContentResolver resolver, long folderId) {
        // 查询笔记数据库，获取指定文件夹下的笔记小部件信息
        Cursor c = resolver.query(Notes.CONTENT_NOTE_URI,
                new String[] { NoteColumns.WIDGET_ID, NoteColumns.WIDGET_TYPE },
                NoteColumns.PARENT_ID + "=?",
                new String[] { String.valueOf(folderId) },
                null);

        // 初始化小部件属性集合为 null
        HashSet<AppWidgetAttribute> set = null;
        // 检查游标是否不为空
        if (c != null) {
            // 将游标移动到第一行
            if (c.moveToFirst()) {
                // 创建一个小部件属性集合
                set = new HashSet<AppWidgetAttribute>();
                do {
                    try {
                        // 创建一个小部件属性对象
                        AppWidgetAttribute widget = new AppWidgetAttribute();
                        // 从游标中获取小部件 ID
                        widget.widgetId = c.getInt(0);
                        // 从游标中获取小部件类型
                        widget.widgetType = c.getInt(1);
                        // 将小部件属性对象添加到集合中
                        set.add(widget);
                    } catch (IndexOutOfBoundsException e) {
                        // 捕获索引越界异常并记录错误日志
                        Log.e(TAG, e.toString());
                    }
                } while (c.moveToNext());
            }
            // 关闭游标
            c.close();
        }
        // 返回小部件属性集合
        return set;
    }

    /**
     * 根据笔记 ID 获取通话号码的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param noteId 笔记的 ID
     * @return 通话号码，如果未找到则返回空字符串
     */
    public static String getCallNumberByNoteId(ContentResolver resolver, long noteId) {
        // 查询数据数据库，获取指定笔记对应的通话号码
        Cursor cursor = resolver.query(Notes.CONTENT_DATA_URI,
                new String [] { CallNote.PHONE_NUMBER },
                CallNote.NOTE_ID + "=? AND " + CallNote.MIME_TYPE + "=?",
                new String [] { String.valueOf(noteId), CallNote.CONTENT_ITEM_TYPE },
                null);

        // 检查游标是否不为空且能移动到第一行
        if (cursor != null && cursor.moveToFirst()) {
            try {
                // 从游标中获取通话号码
                return cursor.getString(0);
            } catch (IndexOutOfBoundsException e) {
                // 捕获索引越界异常并记录错误日志
                Log.e(TAG, "Get call number fails " + e.toString());
            } finally {
                // 关闭游标
                cursor.close();
            }
        }
        // 未找到通话号码，返回空字符串
        return "";
    }

    /**
     * 根据电话号码和通话日期获取笔记 ID 的方法。
     * 
     * @param resolver 内容解析器，用于与内容提供者进行交互
     * @param phoneNumber 电话号码
     * @param callDate 通话日期
     * @return 笔记的 ID，如果未找到则返回 0
     */
    public static long getNoteIdByPhoneNumberAndCallDate(ContentResolver resolver, String phoneNumber, long callDate) {
        // 查询数据数据库
        Cursor cursor = resolver.query(Notes.CONTENT_DATA_URI,
                new String [] { CallNote.NOTE_ID },
                CallNote.CALL_DATE + "=? AND " + CallNote.MIME_TYPE + "=? AND PHONE_NUMBERS_EQUAL("
                + CallNote.PHONE_NUMBER + ",?)",
                new String [] { String.valueOf(callDate), CallNote.CONTENT_ITEM_TYPE, phoneNumber },
                null);

        if (cursor != null) {
            if (cursor.moveToFirst()) {
                try {
                    return cursor.getLong(0);
                } catch (IndexOutOfBoundsException e) {
                    Log.e(TAG, "Get call note id fails " + e.toString());
                }
            }
            cursor.close();
        }
        return 0;
    }

    public static String getSnippetById(ContentResolver resolver, long noteId) {
        Cursor cursor = resolver.query(Notes.CONTENT_NOTE_URI,
                new String [] { NoteColumns.SNIPPET },
                NoteColumns.ID + "=?",
                new String [] { String.valueOf(noteId)},
                null);

        if (cursor != null) {
            String snippet = "";
            if (cursor.moveToFirst()) {
                snippet = cursor.getString(0);
            }
            cursor.close();
            return snippet;
        }
        throw new IllegalArgumentException("Note is not found with id: " + noteId);
    }

    public static String getFormattedSnippet(String snippet) {
        if (snippet != null) {
            snippet = snippet.trim();
            int index = snippet.indexOf('\n');
            if (index != -1) {
                snippet = snippet.substring(0, index);
            }
        }
        return snippet;
    }
}
